@@ -25,15 +25,15 @@ GtkWidget *darkerPreview, *lighterPreview, *middlePreview;
 GtkWidget *allOrSell, *dlg;
 GtkWidget *plusSatPreview, *SatPreview, *minusSatPreview;
 
-struct
+struct 
 {
-  GtkWidget *bna,
-    *palette,
-    *rough,
-    *range,
-    *show,
-    *lnd,
-    *pixelsBy,
+  GtkWidget *bna, 
+    *palette, 
+    *rough, 
+    *range, 
+    *show, 
+    *lnd, 
+    *pixelsBy, 
     *frameSelect,
     *satur;
 } fpFrames;
@@ -52,10 +52,10 @@ ReducedImage *reduced;
 GtkWidget *fp_create_bna(void)
 {
   GtkWidget *frame, *blabel, *alabel, *bframe, *aframe, *table;
-
+  
   Create_A_Preview(&origPreview,&bframe, reduced->width, reduced->height);
   Create_A_Preview(&curPreview,&aframe, reduced->width, reduced->height);
-
+  
   frame = gtk_frame_new ("Before And After");
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
   gtk_container_border_width (GTK_CONTAINER (frame), 5);
@@ -73,9 +73,9 @@ GtkWidget *fp_create_bna(void)
   gtk_container_border_width(GTK_CONTAINER(table),10);
   gtk_table_set_row_spacings(GTK_TABLE(table),0);
   gtk_table_set_col_spacings(GTK_TABLE(table),20);
-
+  
   gtk_container_add (GTK_CONTAINER (frame), table);
-
+  
   gtk_table_attach(GTK_TABLE(table),blabel,0,1,0,1,
 		   0,
 		   GTK_EXPAND|GTK_FILL,
@@ -109,7 +109,9 @@ GtkWidget *fp_create_circle_palette(void)
   GtkWidget *mVbox, *mFrame;
   GtkWidget *centerVbox, *centerFrame;
 
-  frame = gtk_frame_new ("Palette Circle");
+  GtkWidget *win;
+
+  frame = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
   gtk_container_border_width (GTK_CONTAINER (frame), 5);
   gtk_widget_show (frame);
@@ -137,23 +139,27 @@ GtkWidget *fp_create_circle_palette(void)
   Create_A_Table_Entry(&yVbox,yFrame,"Yellow:");
   Create_A_Table_Entry(&mVbox,mFrame,"Magenta:");
   Create_A_Table_Entry(&centerVbox,centerFrame,"Current:");
-
-  gtk_table_attach( GTK_TABLE(table), rVbox,8,11,4,7,
+  
+  gtk_table_attach( GTK_TABLE(table), rVbox,8,11,4,7, 
 		    GTK_EXPAND , GTK_EXPAND,0,0);
-  gtk_table_attach( GTK_TABLE(table), gVbox,2,5,0,3,
+  gtk_table_attach( GTK_TABLE(table), gVbox,2,5,0,3, 
 		    GTK_EXPAND, GTK_EXPAND,0,0);
-  gtk_table_attach( GTK_TABLE(table), bVbox,2,5,8,11,
+  gtk_table_attach( GTK_TABLE(table), bVbox,2,5,8,11, 
 		    GTK_EXPAND, GTK_EXPAND,0,0);
-  gtk_table_attach( GTK_TABLE(table), cVbox,0,3,4,7,
+  gtk_table_attach( GTK_TABLE(table), cVbox,0,3,4,7, 
 		    GTK_EXPAND, GTK_EXPAND,0,0);
-  gtk_table_attach( GTK_TABLE(table), yVbox,6,9,0,3,
+  gtk_table_attach( GTK_TABLE(table), yVbox,6,9,0,3, 
 		    GTK_EXPAND, GTK_EXPAND,0,0);
-  gtk_table_attach( GTK_TABLE(table), mVbox,6,9,8,11,
+  gtk_table_attach( GTK_TABLE(table), mVbox,6,9,8,11, 
 		    GTK_EXPAND, GTK_EXPAND,0,0);
-  gtk_table_attach( GTK_TABLE(table), centerVbox,4,7,4,7,
+  gtk_table_attach( GTK_TABLE(table), centerVbox,4,7,4,7, 
 		    GTK_EXPAND, GTK_EXPAND,0,0);
 
-  return frame;
+  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title(GTK_WINDOW(win),"Hue Variations");
+  gtk_container_add(GTK_CONTAINER(win),frame);
+ 
+  return win;
 }
 
 GtkWidget *fp_create_rough(void)
@@ -166,25 +172,24 @@ GtkWidget *fp_create_rough(void)
   gtk_container_border_width (GTK_CONTAINER (frame), 5);
   gtk_widget_show (frame);
 
-  data = gtk_adjustment_new (Current.Rough,
+  data = gtk_adjustment_new (Current.Rough, 
 			     0, 1.0, 0.05, 0.01, 0.0);
   Current.roughnessScale=
   scale = gtk_hscale_new (GTK_ADJUSTMENT (data));
-
-  gtk_widget_set_usize (scale, 120, 0);
+   
+  gtk_widget_set_usize (scale, 60, 0);
   gtk_scale_set_value_pos (GTK_SCALE (scale), GTK_POS_TOP);
   gtk_scale_set_digits (GTK_SCALE (scale), 2);
   gtk_signal_connect (GTK_OBJECT (data), "value_changed",
 		      (GtkSignalFunc) fp_scale_update,
-		      &Current.Rough);
+		      &Current.Rough);      
   gtk_widget_show (scale);
 
   vbox=gtk_vbox_new(FALSE,5);
-  gtk_container_border_width(GTK_CONTAINER(vbox),10);
   gtk_widget_show(vbox);
-  gtk_box_pack_start (GTK_BOX(vbox), scale,TRUE,TRUE,0);
+  gtk_box_pack_start (GTK_BOX(vbox), scale,TRUE,TRUE,5);
   gtk_container_add (GTK_CONTAINER(frame), vbox);
-
+  
   return frame;
 }
 
@@ -196,64 +201,111 @@ GtkWidget *fp_create_range(void)
   frame = gtk_frame_new ("Affected Range");
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
   gtk_container_border_width (GTK_CONTAINER (frame), 5);
-
+   
   gtk_widget_show (frame);
 
   /********THE THREE RANGES*************/
   vbox=gtk_vbox_new(FALSE,5);
   gtk_container_add(GTK_CONTAINER(frame),vbox);
-  gtk_container_border_width(GTK_CONTAINER(vbox),5);
+  gtk_container_border_width(GTK_CONTAINER(vbox),5);  
   gtk_widget_show(vbox);
 
-  group=Button_In_A_Box(vbox,group,"Shadows",
+  group=Button_In_A_Box(vbox,group,"Shadows", 
 			 (GtkSignalFunc) fp_change_current_range,
 			 ShMidHi+SHADOWS,
 			 Current.Range==SHADOWS);
-  group=Button_In_A_Box(vbox,group,"Midtones",
+  group=Button_In_A_Box(vbox,group,"Midtones", 
 			 (GtkSignalFunc) fp_change_current_range,
 			 ShMidHi+MIDTONES,
 			 Current.Range==MIDTONES);
-  group=Button_In_A_Box(vbox,group,"Highlights",
+  group=Button_In_A_Box(vbox,group,"Highlights", 
 			 (GtkSignalFunc) fp_change_current_range,
 			 ShMidHi+HIGHLIGHTS,
 			 Current.Range==HIGHLIGHTS);
   return frame;
 }
 
+  /********THE THREE RANGES*************/
+  /********THE THREE RANGES*************/
+  /********THE THREE RANGES*************/
+  /********THE THREE RANGES*************/
+  /********THE THREE RANGES*************/
+GtkWidget *fp_create_control(void)
+{
+  GtkWidget *frame, *table, *box;
+  GSList *group=NULL;
+
+  frame = gtk_frame_new ("Windows");
+  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
+  gtk_container_border_width (GTK_CONTAINER (frame), 5);
+   
+  gtk_widget_show (frame);
+
+  /********THE THREE RANGES*************/
+  box=gtk_vbox_new(FALSE,5);
+  gtk_container_add(GTK_CONTAINER(frame),box);
+  gtk_container_border_width(GTK_CONTAINER(box),5);  
+  gtk_widget_show(box);
+
+  Check_Button_In_A_Box(box,"Hue",
+			(GtkSignalFunc) fp_show_hide_frame,
+			fpFrames.palette,
+			Current.VisibleFrames&HUE);
+  Check_Button_In_A_Box(box,"Saturation",
+			(GtkSignalFunc) fp_show_hide_frame,
+			fpFrames.satur,
+			Current.VisibleFrames&SATURATION);
+  Check_Button_In_A_Box(box,"Value",
+			(GtkSignalFunc) fp_show_hide_frame,
+			fpFrames.lnd,
+			Current.VisibleFrames&VALUE);
+  Check_Button_In_A_Box(box,"Advanced",
+			(GtkSignalFunc) fp_show_hide_frame,
+			AW.window,
+			FALSE); 
+			
+  return frame;
+}
+
 GtkWidget *fp_create_lnd()
 {
   GtkWidget *frame, *table, *lighterFrame, *middleFrame, *darkerFrame;
   GtkWidget *lighterVbox, *middleVbox, *darkerVbox;
+  GtkWidget *win;
 
   Create_A_Preview(&lighterPreview,&lighterFrame, reduced->width, reduced->height);
   Create_A_Preview(&middlePreview,&middleFrame, reduced->width, reduced->height);
   Create_A_Preview(&darkerPreview,&darkerFrame, reduced->width, reduced->height);
-
+ 
   Create_A_Table_Entry(&lighterVbox,lighterFrame,"Lighter:");
   Create_A_Table_Entry(&middleVbox,middleFrame,"Current:");
   Create_A_Table_Entry(&darkerVbox,darkerFrame,"Darker:");
 
-  frame = gtk_frame_new ("Lighter and Darker");
+  frame = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
   gtk_container_border_width (GTK_CONTAINER (frame), 5);
   gtk_widget_show (frame);
 
-  table = gtk_table_new(11,1,FALSE);
+  table = gtk_table_new(1,11,FALSE);
   gtk_container_border_width(GTK_CONTAINER(table),0);
   gtk_table_set_row_spacings(GTK_TABLE(table),0);
   gtk_table_set_col_spacings(GTK_TABLE(table),3);
   gtk_widget_show (table);
 
-
-  gtk_table_attach( GTK_TABLE(table), lighterVbox,0,1,0,3,
+    
+  gtk_table_attach( GTK_TABLE(table), lighterVbox,0,3,0,1, 
 		    GTK_EXPAND , GTK_EXPAND,0,0);
-  gtk_table_attach( GTK_TABLE(table), middleVbox,0,1,4,7,
+  gtk_table_attach( GTK_TABLE(table), middleVbox,4,7,0,1, 
 		    GTK_EXPAND, GTK_EXPAND,0,0);
-  gtk_table_attach( GTK_TABLE(table), darkerVbox,0,1,8,11,
+  gtk_table_attach( GTK_TABLE(table), darkerVbox,8,11,0,1, 
 		    GTK_EXPAND, GTK_EXPAND,0,0);
   gtk_container_add (GTK_CONTAINER (frame), table);
 
-  return frame;
+  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title(GTK_WINDOW(win),"Value Variations");
+  gtk_container_add(GTK_CONTAINER(win),frame);
+ 
+  return win;
 }
 
 GtkWidget *fp_create_msnls()
@@ -261,36 +313,44 @@ GtkWidget *fp_create_msnls()
 
   GtkWidget *frame, *table, *lessFrame, *middleFrame, *moreFrame;
   GtkWidget *lessVbox, *middleVbox, *moreVbox;
-
+  GtkWidget *win;
+ 
   Create_A_Preview(&minusSatPreview,&lessFrame, reduced->width, reduced->height);
   Create_A_Preview(&SatPreview,&middleFrame, reduced->width, reduced->height);
   Create_A_Preview(&plusSatPreview,&moreFrame, reduced->width, reduced->height);
-
-  Create_A_Table_Entry(&moreVbox,moreFrame,"More Saturated:");
+ 
+  Create_A_Table_Entry(&moreVbox,moreFrame,"More Sat:");
   Create_A_Table_Entry(&middleVbox,middleFrame,"Current:");
-  Create_A_Table_Entry(&lessVbox,lessFrame,"Less Saturated:");
+  Create_A_Table_Entry(&lessVbox,lessFrame,"Less Sat:");
 
-  frame = gtk_frame_new ("Saturation");
+  frame = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
   gtk_container_border_width (GTK_CONTAINER (frame), 5);
+  gtk_widget_show(frame);
 
-  table = gtk_table_new(11,1,FALSE);
+  table = gtk_table_new(1,11,FALSE);
   gtk_container_border_width(GTK_CONTAINER(table),0);
   gtk_table_set_row_spacings(GTK_TABLE(table),0);
   gtk_table_set_col_spacings(GTK_TABLE(table),3);
   gtk_widget_show (table);
 
-
-  gtk_table_attach( GTK_TABLE(table), moreVbox,0,1,0,3,
+    
+  gtk_table_attach( GTK_TABLE(table), moreVbox,0,3,0,1, 
 		    GTK_EXPAND , GTK_EXPAND,0,0);
-  gtk_table_attach( GTK_TABLE(table), middleVbox,0,1,4,7,
+  gtk_table_attach( GTK_TABLE(table), middleVbox,4,7,0,1, 
 		    GTK_EXPAND, GTK_EXPAND,0,0);
-  gtk_table_attach( GTK_TABLE(table), lessVbox,0,1,8,11,
+  gtk_table_attach( GTK_TABLE(table), lessVbox,8,11,0,1, 
 		    GTK_EXPAND, GTK_EXPAND,0,0);
   gtk_container_add (GTK_CONTAINER (frame), table);
 
-  return frame;
+  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title(GTK_WINDOW(win),"Saturation Variations");
+  gtk_container_add(GTK_CONTAINER(win),frame);
+ 
+  return win;
 }
+
+
 GtkWidget *fp_create_pixels_select_by()
 {
   GtkWidget *frame, *vbox;
@@ -298,24 +358,24 @@ GtkWidget *fp_create_pixels_select_by()
 
   frame = gtk_frame_new ("Select Pixels By");
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
-  gtk_container_border_width (GTK_CONTAINER (frame), 5);
+  gtk_container_border_width (GTK_CONTAINER (frame), 5);  
+  gtk_widget_show(frame);
 
   vbox=gtk_vbox_new(FALSE,5);
   gtk_container_add(GTK_CONTAINER(frame),vbox);
-  gtk_container_border_width(GTK_CONTAINER(vbox),5);
+  gtk_container_border_width(GTK_CONTAINER(vbox),5);  
   gtk_widget_show(vbox);
 
-  group=Button_In_A_Box(vbox,group,"Hue",
+  group=Button_In_A_Box(vbox,group,"Hue", 
 			 (GtkSignalFunc) fp_change_current_pixels_by,
 			 HueSatVal+0,
 			 Current.ValueBy==BY_HUE);
-
-  group=Button_In_A_Box(vbox,group,"Saturation",
+  group=Button_In_A_Box(vbox,group,"Saturation", 
 			 (GtkSignalFunc) fp_change_current_pixels_by,
 			 HueSatVal+1,
 			 Current.ValueBy==BY_SAT);
 
-  group=Button_In_A_Box(vbox,group,"Value",
+  group=Button_In_A_Box(vbox,group,"Value", 
 			 (GtkSignalFunc) fp_change_current_pixels_by,
 			 HueSatVal+2,
 			 Current.ValueBy==BY_VAL);
@@ -333,21 +393,21 @@ GtkWidget *fp_create_show()
 
   vbox=gtk_vbox_new(FALSE,5);
   gtk_container_add(GTK_CONTAINER(frame),vbox);
-  gtk_container_border_width(GTK_CONTAINER(vbox),5);
+  gtk_container_border_width(GTK_CONTAINER(vbox),5);  
   gtk_widget_show(vbox);
 
 
-  group=Button_In_A_Box(vbox,group,"Entire Image",
+  group=Button_In_A_Box(vbox,group,"Entire Image", 
 			 (GtkSignalFunc) fp_entire_image,
 			 &Current.SlctnOnly,
 			 FALSE);
 
-  group=Button_In_A_Box(vbox,group,"Selection Only",
+  group=Button_In_A_Box(vbox,group,"Selection Only", 
 			 (GtkSignalFunc) fp_selection_only,
 			 &Current.SlctnOnly,
 			 TRUE);
 
-  group=Button_In_A_Box(vbox,group,"Selection In Context",
+  group=Button_In_A_Box(vbox,group,"Selection In Context", 
 			 (GtkSignalFunc) fp_selection_in_context,
 			 &Current.SlctnOnly,
 			 FALSE);
@@ -363,7 +423,7 @@ GtkWidget *fp_create_frame_select()
 
   box=gtk_hbox_new(FALSE,15);
   gtk_container_add(GTK_CONTAINER(frame),box);
-  gtk_container_border_width(GTK_CONTAINER(box),5);
+  gtk_container_border_width(GTK_CONTAINER(box),5);  
   gtk_widget_show(box);
 
   Check_Button_In_A_Box (box,"CirclePalette",
@@ -397,14 +457,14 @@ void Create_A_Preview (GtkWidget  **preview,
 
 
 GSList *  Button_In_A_Box        (GtkWidget  *vbox,
-					  GSList     *group,
-					  guchar     *label,
-					  GtkSignalFunc function,
-					  gpointer   data,
-					  int        clicked)
+				  GSList     *group,
+				  guchar     *label,
+				  GtkSignalFunc function,
+				  gpointer   data,
+				  int        clicked)
 {
   GtkWidget *button;
-
+  
   button=gtk_radio_button_new_with_label(group,label);
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
 		      (GtkSignalFunc) function,
@@ -416,14 +476,11 @@ GSList *  Button_In_A_Box        (GtkWidget  *vbox,
   return gtk_radio_button_group(GTK_RADIO_BUTTON(button));
 }
 
-
-
-
 void Check_Button_In_A_Box       (GtkWidget  *vbox,
-					  guchar     *label,
-					  GtkSignalFunc function,
-					  gpointer   data,
-					  int        clicked)
+				  guchar     *label,
+				  GtkSignalFunc function,
+				  gpointer   data,
+				  int        clicked)
 {
   GtkWidget *button;
 
@@ -450,7 +507,7 @@ void Create_A_Table_Entry    (GtkWidget  **box,
   label = gtk_label_new(description);
   gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
   gtk_widget_show (label);
-
+ 
   table=gtk_table_new(2,1,FALSE);
   gtk_widget_show(table);
 
@@ -473,6 +530,29 @@ void Create_A_Table_Entry    (GtkWidget  **box,
     gtk_table_attach(GTK_TABLE(table), SmallerFrame, 0,1,1,2,0,0,0,5);
 }
 
+void
+fp_redraw_all_windows()
+{
+  
+  reduced=Reduce_The_Image(drawable,mask,
+			   Current.PreviewSize,
+			   Current.SlctnOnly);
+  Adjust_Preview_Sizes(reduced->width,reduced->height);
+
+  /*gtk_container_check_resize(GTK_CONTAINER(fpFrames.palette), NULL);*/
+  gtk_widget_draw(fpFrames.palette,NULL);
+  
+  /*gtk_container_check_resize(GTK_CONTAINER(fpFrames.satur), NULL);*/
+  gtk_widget_draw(fpFrames.satur,NULL);
+  
+  /*gtk_container_check_resize(GTK_CONTAINER(fpFrames.lnd), NULL);*/
+  gtk_widget_draw(fpFrames.lnd,NULL);
+  
+  /*gtk_container_check_resize(GTK_CONTAINER(dlg), NULL);*/
+  gtk_widget_draw(dlg,NULL);
+  
+  refreshPreviews(Current.VisibleFrames);
+}
 
 void
 fp_entire_image    (GtkWidget *button,
@@ -481,14 +561,8 @@ fp_entire_image    (GtkWidget *button,
   if (!GTK_TOGGLE_BUTTON(button)->active)
     return;
   Current.SlctnOnly = 0;
-  reduced=Reduce_The_Image(drawable,mask,
-			   Current.PreviewSize,
-			   Current.SlctnOnly);
-  Adjust_Preview_Sizes(reduced->width,reduced->height);
-  refreshPreviews(Current.VisibleFrames);
-  gtk_widget_queue_resize(GTK_WIDGET(dlg));
-  gtk_widget_draw(dlg,NULL);
-}
+  fp_redraw_all_windows();
+} 
 
 
 void
@@ -496,19 +570,13 @@ fp_selection_only  (GtkWidget *button,
 		    gpointer  data )
 {
   static int notFirstTime=0;
-
+  
   if (!(notFirstTime++))  return;
 
   if (!GTK_TOGGLE_BUTTON(button)->active)
     return;
   Current.SlctnOnly = 1;
-  reduced=Reduce_The_Image(drawable,mask,
-			   Current.PreviewSize,
-			   Current.SlctnOnly);
-  Adjust_Preview_Sizes(reduced->width,reduced->height);
-  refreshPreviews(Current.VisibleFrames);
-  gtk_widget_queue_resize(GTK_WIDGET(dlg));
-  gtk_widget_draw(dlg,NULL);
+  fp_redraw_all_windows();
 }
 
 
@@ -519,16 +587,10 @@ fp_selection_in_context  (GtkWidget *button,
   if (!GTK_TOGGLE_BUTTON(button)->active)
     return;
   Current.SlctnOnly = 2;
-  reduced=Reduce_The_Image(drawable,mask,
-			   Current.PreviewSize,
-			   Current.SlctnOnly);
-  Adjust_Preview_Sizes(reduced->width,reduced->height);
-  refreshPreviews(Current.VisibleFrames);
-  gtk_widget_queue_resize(GTK_WIDGET(dlg));
-  gtk_widget_draw(dlg,NULL);
+  fp_redraw_all_windows();
 }
 
-void fp_show_hide_frame (GtkWidget *button,
+void fp_show_hide_frame (GtkWidget *button, 
 			 GtkWidget *frame)
 {
   int prev=Current.VisibleFrames;
@@ -536,7 +598,7 @@ void fp_show_hide_frame (GtkWidget *button,
   if (GTK_TOGGLE_BUTTON(button)->active)
     if (!GTK_WIDGET_VISIBLE(frame)) {
       gtk_widget_show(frame);
-      if (frame==fpFrames.palette)
+      if (frame==fpFrames.palette) 
 	Current.VisibleFrames |= HUE;
       else if (frame==fpFrames.satur)
 	Current.VisibleFrames |= SATURATION;
@@ -545,11 +607,11 @@ void fp_show_hide_frame (GtkWidget *button,
       refreshPreviews(Current.VisibleFrames & ~prev);
     }
     else ;
-
-  else
+    
+  else 
     if (GTK_WIDGET_VISIBLE(frame)) {
       gtk_widget_hide(frame);
-      if (frame==fpFrames.palette)
+      if (frame==fpFrames.palette) 
 	Current.VisibleFrames &= ~HUE;
       else if (frame==fpFrames.satur)
 	Current.VisibleFrames &= ~SATURATION;
@@ -558,7 +620,6 @@ void fp_show_hide_frame (GtkWidget *button,
     }
 }
 
-
 void Adjust_Preview_Sizes(int width, int height)
 {
   gtk_preview_size (GTK_PREVIEW (origPreview),    width, height);
@@ -579,12 +640,12 @@ void Adjust_Preview_Sizes(int width, int height)
 }
 
 
-void
+void      
 selectionMade         (GtkWidget *widget,
 		       gpointer   data)
 {
   Current.Touched[Current.ValueBy] = 1;
-
+  
   if      (!strcmp("Red:",data))     Update_Current_FP( HUE, RED     );
   else if (!strcmp("Green:",data))   Update_Current_FP( HUE, GREEN   );
   else if (!strcmp("Blue:",data))    Update_Current_FP( HUE, BLUE    );
@@ -593,11 +654,11 @@ selectionMade         (GtkWidget *widget,
   else if (!strcmp("Magenta:",data)) Update_Current_FP( HUE, MAGENTA );
   else if (!strcmp("Darker:",data))  Update_Current_FP( VALUE, DOWN );
   else if (!strcmp("Lighter:",data)) Update_Current_FP( VALUE, UP   );
-  else if (!strcmp("More Saturated:",data))
+  else if (!strcmp("More Saturated:",data)) 
     Update_Current_FP( SATURATION, UP );
-  else if (!strcmp("Less Saturated:",data))
+  else if (!strcmp("Less Saturated:",data)) 
     Update_Current_FP( SATURATION, DOWN );
-
+  
   refreshPreviews(Current.VisibleFrames);
 }
 
@@ -605,8 +666,8 @@ selectionMade         (GtkWidget *widget,
 void refreshPreviews(gint which)
 {
   fp_Create_Nudge(nudgeArray);
-  fp_render_preview( origPreview,     NONEATALL,  0       );
-  fp_render_preview( curPreview,      CURRENT,    0       );
+  fp_render_preview  ( origPreview,     NONEATALL,  0       );
+  fp_render_preview  ( curPreview,      CURRENT,    0       );
   if (which & HUE) {
     fp_render_preview( rPreview,        HUE,        RED     );
     fp_render_preview( gPreview,        HUE,        GREEN   );
@@ -666,11 +727,11 @@ fp_change_current_range  (GtkAdjustment *button,
 {
   static FP_Intensity prevValue=MIDTONES;
   static int notFirstTime=0;
-
+  
   if (!(notFirstTime++))  return;
   if (!GTK_TOGGLE_BUTTON(button)->active)  return;
   if (*Intensity == prevValue) return;
-
+ 
   Current.Range = *Intensity;
   refreshPreviews(Current.VisibleFrames);
   if (AW.window != NULL && GTK_WIDGET_VISIBLE (AW.window))
@@ -684,11 +745,11 @@ fp_change_current_pixels_by  (GtkWidget *button,
 {
   int prevValue=VALUE;
   static int notFirstTime=0;
-
+  
   if (!(notFirstTime++))  return;
   if (!GTK_TOGGLE_BUTTON(button)->active)  return;
   if (*valueBy == prevValue) return;
-
+ 
   Current.ValueBy = *valueBy;
   refreshPreviews(Current.VisibleFrames);
   if (AW.window != NULL && GTK_WIDGET_VISIBLE (AW.window) && AW.rangePreview != NULL)
@@ -699,13 +760,13 @@ fp_change_current_pixels_by  (GtkWidget *button,
 
 void
 fp_advanced_call ()
-{
-  if (AW.window!=NULL)
+{ 
+  if (AW.window!=NULL) 
     if(GTK_WIDGET_VISIBLE(AW.window))
       gtk_widget_hide(AW.window);
-    else
+    else 
       gtk_widget_show(AW.window);
-  else
+  else 
     fp_advanced_dialog();
 }
 
@@ -717,18 +778,18 @@ int fp_dialog()
   GtkWidget *show;
   GtkWidget *rough;
   GtkWidget *range;
-  GtkWidget *frameSelect;
   GtkWidget *pixelsBy;
   GtkWidget *satur;
+  GtkWidget *control;
 
   GtkWidget *table;
-  GtkWidget *OKbutton, *CANCELbutton, *OPTIONSbutton, *RESETbutton;
+  GtkWidget *OKbutton, *CANCELbutton, *RESETbutton; 
   GtkWidget *buttonTable;
 
   guchar *color_cube;
   gchar **argv;
   gint argc;
-
+ 
 
   reduced=Reduce_The_Image(drawable,mask,
 			   Current.PreviewSize,
@@ -739,7 +800,7 @@ int fp_dialog()
   argv[0] = g_strdup ("fp");
 
   gtk_init (&argc, &argv);
-  gtk_rc_parse (gimp_gtkrc ());
+  gtk_rc_parse (gimp_gtkrc ());  
 
   gtk_preview_set_gamma (gimp_gamma ());
   gtk_preview_set_install_cmap (gimp_install_cmap ());
@@ -775,13 +836,6 @@ int fp_dialog()
   gtk_widget_show (CANCELbutton);
 
 
-  OPTIONSbutton = gtk_button_new_with_label ("Advanced Options");
-  GTK_WIDGET_SET_FLAGS (OPTIONSbutton, GTK_CAN_DEFAULT);
-  gtk_signal_connect_object (GTK_OBJECT (OPTIONSbutton), "clicked",
-			     (GtkSignalFunc) fp_advanced_call,
-			     NULL);
-  gtk_widget_show (OPTIONSbutton);
-
   RESETbutton = gtk_button_new_with_label ("Reset");
   GTK_WIDGET_SET_FLAGS (RESETbutton, GTK_CAN_DEFAULT);
   gtk_signal_connect_object (GTK_OBJECT (RESETbutton), "clicked",
@@ -789,26 +843,26 @@ int fp_dialog()
 			     NULL);
   gtk_widget_show (RESETbutton);
 
-
-  buttonTable=gtk_table_new(1,7,TRUE);
+  
+  buttonTable=gtk_table_new(1,4,TRUE);
   gtk_container_border_width(GTK_CONTAINER(buttonTable),0);
   gtk_table_set_col_spacings(GTK_TABLE(buttonTable),3);
-
-
+  
+   
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->action_area), buttonTable, TRUE, TRUE, 0);
 
-  gtk_table_attach( GTK_TABLE(buttonTable), OKbutton,0,1,0,1,
+  gtk_table_attach( GTK_TABLE(buttonTable), OKbutton,0,1,0,1, 
 		    GTK_FILL|GTK_EXPAND,0,0,0);
-  gtk_table_attach( GTK_TABLE(buttonTable), CANCELbutton,1,2,0,1,
+  gtk_table_attach( GTK_TABLE(buttonTable), CANCELbutton,1,2,0,1, 
 		    GTK_FILL|GTK_EXPAND,0,0,0);
-  gtk_table_attach( GTK_TABLE(buttonTable), RESETbutton,6,7,0,1,
+  gtk_table_attach( GTK_TABLE(buttonTable), RESETbutton,3,4,0,1, 
 		    GTK_FILL|GTK_EXPAND,0,0,0);
-  gtk_table_attach( GTK_TABLE(buttonTable), OPTIONSbutton,4,6,0,1,
-		    GTK_FILL|GTK_EXPAND,0,0,0);
-
+		   
   gtk_widget_show (buttonTable);
-
+  
   /********************************************************************/
+  
+  fp_advanced_dialog();
 
   fpFrames.bna          = bna          = fp_create_bna();
   fpFrames.rough        = rough        = fp_create_rough();
@@ -817,66 +871,55 @@ int fp_dialog()
   fpFrames.lnd          = lnd          = fp_create_lnd();
   fpFrames.show         = show         = fp_create_show();
   fpFrames.satur        = satur        = fp_create_msnls();
-  fpFrames.frameSelect  = frameSelect  = fp_create_frame_select();
   fpFrames.pixelsBy     = pixelsBy     = fp_create_pixels_select_by();
+                          control      = fp_create_control();
   /********************************************************************/
   /********************   PUT EVRYTHING TOGETHER     ******************/
+  
 
-  table=gtk_table_new(4,4,FALSE);
+  table=gtk_table_new(4,2,FALSE);
   gtk_table_attach(GTK_TABLE(table),
-		   bna,0,1,0,2,
-		   GTK_EXPAND|GTK_FILL,
-		   GTK_EXPAND|GTK_FILL,
-		   0,0);
-  gtk_table_attach(GTK_TABLE(table),
-		   rough,2,3,2,3,
-		   GTK_EXPAND|GTK_FILL,
-		   GTK_EXPAND|GTK_FILL,
-		   0,0);
-  gtk_table_attach(GTK_TABLE(table),
-		   range,1,2,0,2,
-		   GTK_EXPAND|GTK_FILL,
-		   GTK_EXPAND|GTK_FILL,
-		   0,0);
-  gtk_table_attach(GTK_TABLE(table),
-		   frameSelect,0,2,2,3,
-		   GTK_EXPAND|GTK_FILL,
-		   GTK_EXPAND|GTK_FILL,
-		   0,0);
-  gtk_table_attach(GTK_TABLE(table),
-		   palette,0,2,3,4,
-		   GTK_EXPAND|GTK_FILL,
-		   GTK_EXPAND|GTK_FILL,
-		   0,0);
-  gtk_table_attach(GTK_TABLE(table),
-		   lnd,2,3,3,4,
-		   GTK_EXPAND|GTK_FILL,
-		   GTK_EXPAND|GTK_FILL,
-		   0,0);
-   gtk_table_attach(GTK_TABLE(table),
-		   satur,3,4,3,4,
-		   GTK_EXPAND|GTK_FILL,
-		   GTK_EXPAND|GTK_FILL,
-		   0,0);
-  gtk_table_attach(GTK_TABLE(table),
-		   show,2,3,0,2,
-		   GTK_EXPAND|GTK_FILL,
-		   GTK_EXPAND|GTK_FILL,
-		   0,0);
-  gtk_table_attach(GTK_TABLE(table),
-		   pixelsBy,3,4,0,2,
+		   bna,0,2,0,1,
 		   GTK_EXPAND|GTK_FILL,
 		   GTK_EXPAND|GTK_FILL,
 		   0,0);
 
+  gtk_table_attach(GTK_TABLE(table),
+		   control,1,2,1,3,
+		   GTK_EXPAND|GTK_FILL,
+		   GTK_EXPAND|GTK_FILL,
+		   0,0);
+    gtk_table_attach(GTK_TABLE(table),
+		   rough,1,2,3,4,
+		   GTK_EXPAND|GTK_FILL,
+		   GTK_EXPAND|GTK_FILL,
+		   0,0);
+		   
+
+  gtk_table_attach(GTK_TABLE(table),
+		   show,0,1,1,2,
+		   GTK_FILL,
+		   GTK_EXPAND|GTK_FILL,
+		   0,0);
+  gtk_table_attach(GTK_TABLE(table),
+		   range,0,1,2,3,
+		   GTK_EXPAND|GTK_FILL,
+		   GTK_EXPAND|GTK_FILL,
+		   0,0);
+  gtk_table_attach(GTK_TABLE(table),
+		   pixelsBy,0,1,3,4,
+		   GTK_EXPAND|GTK_FILL,
+		   GTK_EXPAND|GTK_FILL,
+		   0,0);
+		  
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG (dlg)->vbox), table, TRUE,TRUE,0);
   gtk_widget_show(table);
   gtk_widget_show (dlg);
-
+  
   refreshPreviews(Current.VisibleFrames);
   gtk_main ();
   gdk_flush ();
-
+  
   return FPint.run;
 }
 
@@ -890,7 +933,7 @@ void fp_advanced_ok()
 }
 
 
-void
+void     
 As_You_Drag           (GtkWidget *button)
 {
   static int notFirstTime=0;
@@ -920,14 +963,8 @@ void
 preview_size_scale_update (GtkAdjustment *adjustment,
 			   float         *scale_val)
 {
-  *scale_val = adjustment->value;
-  reduced=Reduce_The_Image(drawable,mask,
-			   Current.PreviewSize,
-			   Current.SlctnOnly);
-  Adjust_Preview_Sizes(reduced->width,reduced->height);
-  refreshPreviews(Current.VisibleFrames);
-  gtk_widget_queue_resize(GTK_WIDGET(dlg));
-  gtk_widget_draw(dlg,NULL);
+  Current.PreviewSize = adjustment->value;
+  fp_redraw_all_windows();
 }
 
 
@@ -935,36 +972,30 @@ gint fp_advanced_dialog()
 {
   guchar *rangeNames[]={"Shadows:", "Midtones:", "Highlights:"};
   GtkWidget *button;
-  GtkWidget *frame;
+  GtkWidget *frame, *mainvbox;
   GtkObject *smoothnessData;
   GtkWidget *graphFrame, *table, *scale;
   GtkWidget *vbox, *label, *labelTable;
   GtkWidget *optionsFrame;
   int i;
 
-  AW.window=gtk_dialog_new();
+  AW.window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title(GTK_WINDOW(AW.window),"Advanced Filter Pack Options");
-  button = gtk_button_new_with_label ("OK");
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-                      (GtkSignalFunc) fp_advanced_ok,
-                      NULL);
-  gtk_widget_grab_default (button);
-  gtk_widget_show (button);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (AW.window)->action_area), button, TRUE, TRUE, 0);
-
 
+  mainvbox=gtk_hbox_new(FALSE,5);
+  gtk_container_add(GTK_CONTAINER(AW.window),mainvbox);
+  gtk_widget_show(mainvbox);
   /************************************************************/
   frame = gtk_frame_new("Smoothness of Aliasing");
   gtk_container_border_width(GTK_CONTAINER(frame),10);
   gtk_widget_show(frame);
 
-  gtk_box_pack_start(GTK_BOX (GTK_DIALOG (AW.window)->vbox),frame, TRUE, TRUE,0);
+  gtk_box_pack_start(GTK_BOX (mainvbox),frame, TRUE, TRUE,0);
 
   table=gtk_table_new(3,1,FALSE);
   gtk_widget_show(table);
   gtk_container_add(GTK_CONTAINER(frame),table);
-
+ 
   graphFrame=gtk_frame_new(NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (graphFrame), GTK_SHADOW_IN);
   gtk_container_border_width (GTK_CONTAINER(graphFrame),0);
@@ -973,7 +1004,7 @@ gint fp_advanced_dialog()
 		   GTK_EXPAND,
 		   0,
 		   10,10);
-
+  
   vbox=gtk_vbox_new(FALSE,0);
   gtk_widget_show(vbox);
   gtk_container_add(GTK_CONTAINER(graphFrame),vbox);
@@ -983,13 +1014,13 @@ gint fp_advanced_dialog()
   gtk_widget_show(AW.aliasingPreview);
   gtk_box_pack_start(GTK_BOX(vbox),AW.aliasingPreview,TRUE,TRUE,0);
   fp_create_smoothness_graph(AW.aliasingPreview);
-
+  
   AW.rangePreview=gtk_preview_new (GTK_PREVIEW_COLOR);
   gtk_preview_size (GTK_PREVIEW (AW.rangePreview), 256, RANGE_HEIGHT);
   gtk_widget_show(AW.rangePreview);
   gtk_box_pack_start(GTK_BOX(vbox),AW.rangePreview,TRUE,TRUE,0);
   fp_range_preview_spill(AW.rangePreview,Current.ValueBy);
-
+  
 
   labelTable=gtk_table_new(3,4,FALSE);
   gtk_widget_show(labelTable);
@@ -1001,23 +1032,23 @@ gint fp_advanced_dialog()
   for (i=0; i<12; i++)
     {
       label=Current.rangeLabels[i]=gtk_label_new("-");
-      gtk_widget_show(label);
-      gtk_table_attach(GTK_TABLE(labelTable),label,i%4, i%4+1,i/4, i/4+1,
-		       GTK_EXPAND|GTK_FILL,0,5,0);
       if (!(i%4)) {
 	gtk_label_set(GTK_LABEL(label),rangeNames[i/4]);
 	gtk_misc_set_alignment(GTK_MISC(label),0.0,1.0);
       }
+      gtk_widget_show(label);
+      gtk_table_attach(GTK_TABLE(labelTable),label,i%4, i%4+1,i/4, i/4+1,
+		       GTK_EXPAND|GTK_FILL,0,5,0);
     }
   /************************************************************/
   /************************************************************/
   /************************************************************/
 
-
+  
   AW.aliasingGraph = gtk_drawing_area_new();
   gtk_drawing_area_size (GTK_DRAWING_AREA (AW.aliasingGraph),
-	  		 256,
-	 		 15);
+	  		 2*MARGIN + 256,
+	 		 RANGE_HEIGHT);
   gtk_box_pack_start(GTK_BOX(vbox), AW.aliasingGraph, TRUE, TRUE, 0);
   gtk_widget_show(AW.aliasingGraph);
   gtk_widget_set_events(AW.aliasingGraph,RANGE_ADJUST_MASK);
@@ -1033,7 +1064,7 @@ gint fp_advanced_dialog()
 
   smoothnessData = gtk_adjustment_new (Current.Alias, 0, 1.0, 0.05, 0.01, 0.0);
 
-  Current.aliasingScale =
+  Current.aliasingScale = 
   scale=gtk_hscale_new (GTK_ADJUSTMENT (smoothnessData));
   gtk_widget_set_usize (scale, 200, 0);
   gtk_scale_set_digits (GTK_SCALE (scale), 2);
@@ -1041,7 +1072,7 @@ gint fp_advanced_dialog()
   gtk_range_set_update_policy (GTK_RANGE (scale), 0);
   gtk_signal_connect (GTK_OBJECT (smoothnessData), "value_changed",
 		      (GtkSignalFunc) fp_scale_update,
-		      &Current.Alias);
+		      &Current.Alias);      
   gtk_widget_show (scale);
   gtk_table_attach(GTK_TABLE(table),scale,0,1,2,3,0,0,0,15);
 
@@ -1054,35 +1085,29 @@ gint fp_advanced_dialog()
   gtk_container_border_width(GTK_CONTAINER(optionsFrame),10);
   gtk_widget_show(optionsFrame);
 
-  gtk_box_pack_start(GTK_BOX (GTK_DIALOG (AW.window)->vbox),optionsFrame, TRUE, TRUE,0);
-
+  gtk_box_pack_start(GTK_BOX(mainvbox),optionsFrame, TRUE, TRUE,0);
+ 
   table=gtk_table_new(2,2,FALSE);
   gtk_widget_show(table);
   gtk_container_add(GTK_CONTAINER(optionsFrame),table);
-
+ 
   vbox=gtk_vbox_new(FALSE,8);
   gtk_container_border_width(GTK_CONTAINER(vbox),10);
   gtk_widget_show(vbox);
   gtk_table_attach(GTK_TABLE(table),vbox,0,1,0,1,0,0,5,5);
-
+ 
   Check_Button_In_A_Box (vbox,"Preview As You Drag",
 			  (GtkSignalFunc) As_You_Drag,
-			  NULL,TRUE);
-
-  Check_Button_In_A_Box (vbox,"Pixel Selection Menu",
-			  (GtkSignalFunc)fp_show_hide_frame,
-			  fpFrames.pixelsBy,FALSE);
-
-
-  smoothnessData = gtk_adjustment_new (Current.PreviewSize,
-				       50, MAX_PREVIEW_SIZE,
-				       5, 5, 0.0);
-
+			  NULL,TRUE);			
 
   frame=gtk_frame_new("Preview Size");
   gtk_widget_show(frame);
   gtk_container_border_width(GTK_CONTAINER(frame),10);
-
+ 
+  smoothnessData = gtk_adjustment_new (Current.PreviewSize, 
+				       50, MAX_PREVIEW_SIZE, 
+				       5, 5, 0.0);
+				      
   Current.previewSizeScale=
   scale=gtk_hscale_new (GTK_ADJUSTMENT (smoothnessData));
   gtk_container_add(GTK_CONTAINER(frame),scale);
@@ -1092,10 +1117,10 @@ gint fp_advanced_dialog()
   gtk_range_set_update_policy (GTK_RANGE (scale), 0);
   gtk_signal_connect (GTK_OBJECT (smoothnessData), "value_changed",
 		      (GtkSignalFunc) preview_size_scale_update,
-		      &Current.PreviewSize);
+		      &Current.PreviewSize);        
   gtk_widget_show (scale);
   gtk_table_attach(GTK_TABLE(table),frame,0,1,1,2,GTK_FILL,0,5,5);
 
-  gtk_widget_show(AW.window);
   return 1;
 }
+
