@@ -4,32 +4,60 @@
   Michal Gomulinski
 */
 
+/* 	$Id: callbacks.c,v 1.2 1998/06/03 09:46:13 neo Exp $	 */
+
+#ifndef lint
+static char vcid[] = "$Id: callbacks.c,v 1.2 1998/06/03 09:46:13 neo Exp $";
+#endif /* lint */
+
+
 #include<gtk/gtk.h>
 #include<string.h>
 #include<libgimp/gimp.h>
 #include<megawidget/megawidget.h>
+#include<time.h>
+#include<unistd.h>
+#include"preview.h"
 #include"utils.h"
 #include"lsystem.h"
 #include"lsystui.h"
 #include"scales.h"
 #include"callbacks.h"
+#include"gdebug.h"
 
 gdouble color[3];
 gint edit_rule_dialog=FALSE;
 gint edit_lrule_dialog=FALSE;
 gint edit_srule_dialog=FALSE;
 
-void ls_toggle_widget(GtkWidget *, GtkWidget *);
 
+static GtkWidget *ls_rule_edit_new(gchar **);
+static GtkWidget *ls_newname_new(LSinfo *);
+static void     ls_rule_edit_ok_callback(GtkWidget *, gpointer);
+static void     ls_rule_edit_cancel_callback(GtkWidget *, gpointer);
+static GtkWidget *ls_lrule_edit_new(gchar **);
+static void     ls_lrule_edit_ok_callback(GtkWidget *, gpointer);
+static void     ls_lrule_edit_cancel_callback(GtkWidget *, gpointer);
+static GtkWidget *ls_srule_edit_new(gchar **);
+static void     ls_srule_edit_ok_callback(GtkWidget *, gpointer);
+static void     ls_srule_edit_cancel_callback(GtkWidget *, gpointer);
+static void     ls_newname_ok_callback(GtkWidget *, LSinfo *);
+static void     ls_newname_cancel_callback(GtkWidget *, LSinfo *);
+
+/* Dummy function to disable warning about vcid */
+void callbacks_no_warning()
+{
+  (void)vcid;
+}
 
 void ls_update_main_dialog_callback(GtkWidget *w, gpointer p)
 {
   if (selected_lsystem) {
     ls_grab_values();
-    ls_update_rules_dialog();
   }
   selected_lsystem=ls_get_selected_lsystem();
-  if (!(GTK_LIST(lsui.lst)->selection)) {
+  ls_update_rules_dialog();
+  if (!selected_lsystem) {
     gtk_widget_set_sensitive(lsui.ntb, FALSE);
     gtk_widget_set_sensitive(lsui.lsrm, FALSE);
     gtk_widget_set_sensitive(lsui.lscp, FALSE);
@@ -56,10 +84,8 @@ void ls_copy_callback(GtkWidget *w, gpointer o)
   trg=g_malloc0(sizeof(LSinfo));
   src=selected_lsystem;
   ls_lsystem_copy(trg, src);
-  g_list_append(loaded_lsystems, trg);
   selected_lsystem=trg;
-  ls_create_lsystems_list();
-  ls_newname_new( & trg->def.name, &trg->def.fname);
+  ls_newname_new(trg);
 }
 
 void ls_new_callback(GtkWidget *w, gpointer o)
@@ -69,8 +95,6 @@ void ls_new_callback(GtkWidget *w, gpointer o)
   gchar *path_end, *tmp;
   gint path_len;
 
-  loaded_lsystems=g_list_append(loaded_lsystems, new_lsystem);
-
   if (selected_lsystem) {
     path_end=strrchr(selected_lsystem->def.name, '/');
     path_len=path_end - selected_lsystem->def.name + 1;
@@ -82,22 +106,25 @@ void ls_new_callback(GtkWidget *w, gpointer o)
   }
 
   selected_lsystem=new_lsystem;
-  ls_create_lsystems_list();
 
-  dlg=ls_newname_new(&new_lsystem->def.name, &new_lsystem->def.fname);
+  dlg=ls_newname_new(new_lsystem);
 }
 
 void ls_rename_callback(GtkWidget *w, gpointer o)
 {
+  /*  LSinfo *new; */
+
   if (!selected_lsystem) return;
-  ls_newname_new( &selected_lsystem->def.name , &selected_lsystem->def.fname);
+  ls_newname_new( selected_lsystem );
 }
 
-GtkWidget *ls_newname_new(gchar **oldname, gchar **oldfname)
+static GtkWidget *ls_newname_new(LSinfo* lsystem)
 {
   GtkDialog *wind;
   GtkWidget *wgt, *e1, *e2, *b1;
   GtkTable *tbl;
+  gchar **oldname=&lsystem->def.name, 
+    **oldfname=&lsystem->def.fname;
 
   wind=GTK_DIALOG(gtk_dialog_new());
   gtk_window_set_title(&wind->window, "New Name");
@@ -122,25 +149,26 @@ GtkWidget *ls_newname_new(gchar **oldname, gchar **oldfname)
   
   gtk_object_set_user_data(GTK_OBJECT(b1), wind);
   GTK_WIDGET_SET_FLAGS( b1, GTK_CAN_DEFAULT );
-  add_button(wind->action_area, b1, GTK_SIGNAL_FUNC(ls_newname_ok_callback));
+  add_button_d(wind->action_area, b1, GTK_SIGNAL_FUNC(ls_newname_ok_callback), lsystem);
   wgt=gtk_button_new_with_label("Cancel"); 
   
   gtk_object_set_user_data(GTK_OBJECT(wgt), wind);
   GTK_WIDGET_SET_FLAGS( wgt, GTK_CAN_DEFAULT );
-  add_button(wind->action_area, wgt, GTK_SIGNAL_FUNC(ls_newname_cancel_callback));
+  add_button_d(wind->action_area, wgt, GTK_SIGNAL_FUNC(ls_newname_cancel_callback), lsystem);
   gtk_signal_connect(GTK_OBJECT(wind), "destroy", GTK_SIGNAL_FUNC(ls_univ_close), wind);
   gtk_widget_grab_default(b1);
   gtk_widget_show(GTK_WIDGET(wind));
   return GTK_WIDGET(wind);
 }
 
-void ls_newname_ok_callback(GtkWidget *w, gpointer o)
+static void ls_newname_ok_callback(GtkWidget *w, LSinfo *lsystem)
 {
   GtkWidget *wind,
     *entry1, *entry2;
   gchar **name, **fname;
+  GtkWidget *tree, *treeitem;
 
-  if (!w) return;
+  if (!w || !lsystem) return;
   wind=gtk_object_get_user_data(GTK_OBJECT(w));
   if (!wind) return;
   entry1=gtk_object_get_data(GTK_OBJECT(wind), ENTRY1);
@@ -151,7 +179,8 @@ void ls_newname_ok_callback(GtkWidget *w, gpointer o)
 		 "is already used for some other L-System.");
     return; 
   }
-  if (ls_check_lsystem_filename( gtk_entry_get_text(GTK_ENTRY(entry2)) )==DUPLICATE) {
+  if (!lsystem->lab &&  /* If renaming this doesn't matter */
+      ls_check_lsystem_filename( gtk_entry_get_text(GTK_ENTRY(entry2)) )==DUPLICATE) {
     ls_box_error("The file name you have just enetered\n"
 		 "is already used for storing some other L-System.\n"
 		 "Please select different one.");
@@ -175,13 +204,20 @@ void ls_newname_ok_callback(GtkWidget *w, gpointer o)
 		 "Please try again.");
     return;
   }
-
+  
+  loaded_lsystems=g_list_insert_sorted(loaded_lsystems, lsystem, (GCompareFunc)lscmp);
+  if (lsystem->lab) { /* If renaming ... */
+    treeitem=GTK_WIDGET(lsystem->lab)->parent;
+    tree=GTK_WIDGET(treeitem)->parent;
+    gtk_container_remove(GTK_CONTAINER(tree), treeitem);
+  }
+  lsystem->dirty=TRUE;
+  ls_tree_add_lsystem(lsystem);
   gtk_widget_hide(wind);
   gtk_widget_destroy(wind);
-  ls_create_lsystems_list();
 }
 
-void ls_newname_cancel_callback(GtkWidget *w, gpointer o)
+static void ls_newname_cancel_callback(GtkWidget *w, LSinfo *lsystem)
 {
   GtkWidget *window;
 
@@ -191,9 +227,8 @@ void ls_newname_cancel_callback(GtkWidget *w, gpointer o)
     gtk_widget_hide(window);
     gtk_widget_destroy(window);
   }
-  loaded_lsystems=g_list_remove(loaded_lsystems, selected_lsystem);
-  lsinfo_free(selected_lsystem);
-  ls_create_lsystems_list();
+  if (lsystem) lsinfo_free(lsystem);
+  /* ls_create_lsystems_list(); */
 }
 
 void ls_reread_callback(GtkWidget *w, gpointer o)
@@ -202,10 +237,18 @@ void ls_reread_callback(GtkWidget *w, gpointer o)
 
 void ls_delete_callback(GtkWidget *w, gpointer o)
 {
+  GtkWidget *tree, *treeitem;
+  LSinfo *for_deletion=selected_lsystem;
+
   if (!selected_lsystem) return;
   loaded_lsystems=g_list_remove(loaded_lsystems, selected_lsystem);
-  lsinfo_free(selected_lsystem);
-  ls_create_lsystems_list();
+  treeitem=GTK_WIDGET(selected_lsystem->lab)->parent;
+  tree=GTK_WIDGET(treeitem)->parent;
+  gtk_container_remove(GTK_CONTAINER(tree), treeitem);
+  unlink(for_deletion->def.fname);
+
+  lsinfo_free(for_deletion);
+  /* ls_create_lsystems_list(); */
 }
 
 
@@ -214,7 +257,7 @@ void ls_delete_callback(GtkWidget *w, gpointer o)
 ****************** Generic rule list manipulations
 */
 
-GtkWidget *ls_generic_rule_edit_new(gchar **rstr, GtkSignalFunc ok, GtkSignalFunc cancel)
+static GtkWidget *ls_generic_rule_edit_new(gchar **rstr, GtkSignalFunc ok, GtkSignalFunc cancel)
 {
   GtkDialog *ret;
   GtkWidget *wgt, *box, *e1, *e2, *b1;
@@ -270,7 +313,7 @@ GtkWidget *ls_generic_rule_edit_new(gchar **rstr, GtkSignalFunc ok, GtkSignalFun
   return GTK_WIDGET(ret);
 }
 
-void ls_generic_rule_edit_ok_callback(GtkWidget *w, gchar **rules, 
+static void ls_generic_rule_edit_ok_callback(GtkWidget *w, gchar **rules, 
 				      gint *rulcount, GtkWidget *rullist)
 {
   GtkWidget *edit_dialog=gtk_object_get_user_data(GTK_OBJECT(w));
@@ -299,6 +342,7 @@ void ls_generic_rule_edit_ok_callback(GtkWidget *w, gchar **rules,
     g_free(*rule);
     (*rule)=buf;
   } else (*rule)=buf;
+  rules_changed=TRUE;
 
   gtk_widget_hide(edit_dialog);
   /*  gtk_widget_destroy(edit_dialog); */
@@ -306,7 +350,7 @@ void ls_generic_rule_edit_ok_callback(GtkWidget *w, gchar **rules,
   gtk_widget_set_sensitive(rullist, TRUE);
 }
 
-void ls_generic_rule_edit_cancel_callback(GtkWidget *w, gchar **rules, 
+static void ls_generic_rule_edit_cancel_callback(GtkWidget *w, gchar **rules, 
 				  gint *rulcount, GtkWidget *rullist)
 {
   GtkWidget *edit_dialog=gtk_object_get_user_data(GTK_OBJECT(w));
@@ -371,9 +415,10 @@ void ls_remove_rule_callback(GtkWidget *w, gpointer o)
     rules[i-1]=rules[i];
   selected_lsystem->def.rulnum--;
   ls_clist_remove_selected(GTK_CLIST(lsui.rules));
+  rules_changed=TRUE;
 }
 
-GtkWidget *ls_rule_edit_new(gchar **rstr)
+static GtkWidget *ls_rule_edit_new(gchar **rstr)
 {
   edit_rule_dialog=TRUE;
   gtk_widget_set_sensitive(lsui.ruleadd, FALSE);
@@ -383,7 +428,7 @@ GtkWidget *ls_rule_edit_new(gchar **rstr)
 				   GTK_SIGNAL_FUNC(ls_rule_edit_cancel_callback));
 } 
 
-void ls_rule_edit_ok_callback(GtkWidget *w, gpointer o)
+static void ls_rule_edit_ok_callback(GtkWidget *w, gpointer o)
 {
   edit_rule_dialog=FALSE;
   gtk_widget_set_sensitive(lsui.ruleadd, TRUE);
@@ -394,7 +439,7 @@ void ls_rule_edit_ok_callback(GtkWidget *w, gpointer o)
 				   lsui.rules);
 }
 
-void ls_rule_edit_cancel_callback(GtkWidget *w, gpointer o)
+static void ls_rule_edit_cancel_callback(GtkWidget *w, gpointer o)
 {
   edit_rule_dialog=FALSE;
   gtk_widget_set_sensitive(lsui.ruleadd, TRUE);
@@ -454,9 +499,10 @@ void ls_remove_lrule_callback(GtkWidget *w, gpointer o)
     rules[i-1]=rules[i];
   selected_lsystem->def.lrulnum--;
   ls_clist_remove_selected(GTK_CLIST(lsui.lrules));
+  rules_changed=TRUE;
 }
 
-GtkWidget *ls_lrule_edit_new(gchar **rstr)
+static GtkWidget *ls_lrule_edit_new(gchar **rstr)
 {
   edit_lrule_dialog=TRUE;
   gtk_widget_set_sensitive(lsui.lruleadd, FALSE);
@@ -466,7 +512,7 @@ GtkWidget *ls_lrule_edit_new(gchar **rstr)
 				   GTK_SIGNAL_FUNC(ls_lrule_edit_cancel_callback));
 } 
 
-void ls_lrule_edit_ok_callback(GtkWidget *w, gpointer o)
+static void ls_lrule_edit_ok_callback(GtkWidget *w, gpointer o)
 {
   edit_lrule_dialog=FALSE;
   gtk_widget_set_sensitive(lsui.lruleadd, TRUE);
@@ -477,7 +523,7 @@ void ls_lrule_edit_ok_callback(GtkWidget *w, gpointer o)
 				   lsui.lrules);
 }
 
-void ls_lrule_edit_cancel_callback(GtkWidget *w, gpointer o)
+static void ls_lrule_edit_cancel_callback(GtkWidget *w, gpointer o)
 {
   edit_lrule_dialog=FALSE;
   gtk_widget_set_sensitive(lsui.lruleadd, TRUE);
@@ -519,6 +565,7 @@ void ls_remove_srule_callback(GtkWidget *w, gpointer o)
     rules[i-1]=rules[i];
   selected_lsystem->prm.srulnum--;
   ls_clist_remove_selected(GTK_CLIST(lsui.srules));
+  rules_changed=TRUE;
 }
 
 void ls_edit_srule_callback(GtkWidget *w, gpointer o)
@@ -535,7 +582,7 @@ void ls_edit_srule_callback(GtkWidget *w, gpointer o)
 }
 
 
-GtkWidget *ls_srule_edit_new(gchar **srulestr)
+static GtkWidget *ls_srule_edit_new(gchar **srulestr)
 {
   GtkDialog *ret;
   GtkTable *tbl;
@@ -660,13 +707,15 @@ GtkWidget *ls_srule_edit_new(gchar **srulestr)
     (*srulestr)[1]=0;
     gtk_entry_set_text(e1, (*srulestr));
     (*srulestr)[1]=':';
-    if (vals->brush_spc >= 0)
+    if (vals->brush_spc >= 0) {
+      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(enspc), TRUE);
       gtk_entry_set_int(e2, vals->brush_spc);
-    else
+    } else
       gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(enspc), FALSE);
-    if (vals->opacity >=0) 
+    if (vals->opacity >=0) {
+      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(enopacity), TRUE);
       gtk_entry_set_int(e3, vals->opacity);
-    else
+    } else
       gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(enopacity), FALSE);
     gtk_entry_set_int(e4, vals->turn_angle);
     gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(encolor), 
@@ -688,7 +737,7 @@ GtkWidget *ls_srule_edit_new(gchar **srulestr)
   return GTK_WIDGET(ret);
 }
 
-void ls_srule_edit_ok_callback(GtkWidget *w, gpointer o)
+static void ls_srule_edit_ok_callback(GtkWidget *w, gpointer o)
 {
   GtkWidget *wind, *chrentry, *spcentry, *opcentry, 
     *brchoice, *encolor, *enopacity, *enspc;
@@ -744,9 +793,10 @@ void ls_srule_edit_ok_callback(GtkWidget *w, gpointer o)
   gtk_widget_hide(wind);
   gtk_widget_set_sensitive(lsui.srules, TRUE);
   gtk_widget_set_sensitive(lsui.lst, TRUE);
+  rules_changed=TRUE;
 }
 
-void ls_srule_edit_cancel_callback(GtkWidget *w, gpointer o)
+static void ls_srule_edit_cancel_callback(GtkWidget *w, gpointer o)
 {
   GtkWidget *wind;
   gchar **srule;
@@ -783,6 +833,16 @@ void ls_toggle_widget(GtkWidget *tbutton, GtkWidget *what)
     gtk_widget_set_sensitive(what, TRUE);
 }
 
+void ls_autoscale_toggle(GtkWidget *x, gpointer a)
+{
+  if (GTK_WIDGET_IS_SENSITIVE(GTK_WIDGET(lsui.startx))) {
+    if (selected_lsystem) selected_lsystem->prm.autoscale=TRUE;
+  } else {
+    if (selected_lsystem) selected_lsystem->prm.autoscale=FALSE;
+  }
+  rules_changed=TRUE;
+}
+
 void ls_univ_close(GtkWidget *w, GtkWidget *what)
 {
    if (what) { 
@@ -791,12 +851,29 @@ void ls_univ_close(GtkWidget *w, GtkWidget *what)
    }
 }
 
+void     ls_new_seed_callback(GtkWidget *w, gpointer data)
+{
+  if (!selected_lsystem) return;
+
+  rules_changed=TRUE;
+  selected_lsystem->prm.seed=time(NULL);
+  ls_grab_values();
+  ls_draw_preview();
+}
 
 /*
 ******************************************************
 ********* Main window Callbacks
 */
 
+void ls_preview_callback(GtkWidget *w, gpointer data)
+{
+  if (!selected_lsystem) return;
+  if (gimp_debug()) g_print("Previewing...\n");
+  ls_grab_values();
+  ls_draw_preview();
+}
+
 void ls_cancel_callback(GtkWidget *w, gpointer *data)
 {
   do_draw=FALSE;
@@ -807,7 +884,6 @@ void ls_ok_callback(GtkWidget *w, gpointer *data)
 {
   do_draw=TRUE;
   ls_grab_values();
-  ls_write_all();
   gtk_main_quit();
 }
 
