@@ -7,7 +7,7 @@
 
    Warsaw, June 1997
    Birmingham, October, November 1997
-   Warsaw, February 1998
+   Warsaw, Mai 1998
  */
 
 #include<stdlib.h>
@@ -22,6 +22,7 @@
 #include<dirent.h>
 #include<glib.h>
 #include<libgimp/gimp.h>
+#include<values.h>
 #include"lsystem.h"
 #include"gdebug.h"
 
@@ -36,21 +37,23 @@ gint32 image;
 gint32 drawable;
 guint image_wid;
 guint image_hei;
+guint image_ox;
+guint image_oy;
 gchar *lsystems_home_path;
 gchar last_ls_name[256];
 
+Position maxpos, minpos;
+
 /* 
    Version identification string: 
-   -$Id: lsystem.c,v 1.1 1998/03/27 19:38:09 neo Exp $- 
+   -$Id: lsystem.c,v 1.2 1998/06/03 09:46:15 neo Exp $- 
 */
 
 #ifndef lint
-static char vcid[] = "$Id: lsystem.c,v 1.1 1998/03/27 19:38:09 neo Exp $";
+static char vcid[] = "$Id: lsystem.c,v 1.2 1998/06/03 09:46:15 neo Exp $";
 #endif /* lint */
 
 
-static Path *new_path (Position);
-static Path *add_point (Path *, Position);
 static void flush_path (Path *);
 
 static Context *get_drawing_context ();
@@ -60,11 +63,10 @@ static void change_brush (gchar *);
 static void change_spacing (gint);
 static void change_opacity (gint);
 static void change_color (guchar[3]);
-static gfloat rand_norm (gfloat);	/* Gaussian generator N(0,param) */
-static void free_slut (Rparams *);
 GList *ls_load_all (GList * list);
 GList *parse_lsystem_path ();
 
+
 static void run ();
 static void query ();
 static gint ls_display_dialog ();
@@ -97,7 +99,9 @@ static void query ()
 	       "rules given by the user and then renders it on your image. "
 		   "Several (possibly) useful options are available. These "
 		  "include: color and brush changes, separate set of rules "
-			  "applied just before rendering and others.",
+			  "applied just before rendering and others."
+			  "Setting a 'gimp_debug' variable in gimprc file  "
+			  "(to a 'on') enables lots of debug information. ",
 			  "Michal Gomulinski",
 			  "Michal Gomulinski",
 			  "1997,1998",
@@ -121,17 +125,20 @@ void run (gchar * name,
   gchar *compiled;
   GParam *rets;
   gint nrets;
+  gdouble kx, ky, k, keep_x=0, keep_y=0;
+  gchar *ls_to_draw;
+  GList *p=NULL;
 
   (void)vcid;
   if (gimp_debug_get_state("lsystem")) {
     g_print("--Debug mode-- Press any key after starting debugger.\n");
     getchar();
   }
-  srandom(time(NULL));
 
   run_mode = param[0].data.d_int32;
   image=param[1].data.d_image;
   drawable=param[2].data.d_drawable;
+  ls_to_draw=param[3].data.d_string;
 
   *nreturn_vals = 1;
   *return_vals = values;
@@ -140,22 +147,47 @@ void run (gchar * name,
   values[0].data.d_status = status;
 
   ls_get_brushes();
-  image_wid=gimp_drawable_width(drawable);
-  image_hei=gimp_drawable_height(drawable);
+  rets=gimp_run_procedure("gimp_selection_bounds", &nrets,
+			  PARAM_IMAGE, image,
+			  PARAM_END);
+  if ((rets[0].data.d_status == STATUS_SUCCESS) && (rets[1].data.d_int32 > 0)) {
+    image_ox=rets[2].data.d_int32;
+    image_oy=rets[3].data.d_int32;
+    image_wid=rets[4].data.d_int32 - image_ox;
+    image_hei=rets[5].data.d_int32 - image_oy;
+  } else {
+    image_ox=0;
+    image_oy=0;
+    image_wid=gimp_drawable_width(drawable);
+    image_hei=gimp_drawable_height(drawable);
+  }
+  gimp_destroy_params(rets, nrets);
 
   switch (run_mode) {
   case RUN_INTERACTIVE:
     gimp_get_data(PLUG_IN_NAME, &last_ls_name);
     if (!ls_display_dialog ())
-      return;
+      status=STATUS_EXECUTION_ERROR;
     break;
   case RUN_NONINTERACTIVE:
-    g_print ("NON Interactive: %s\nNot supported yet!\n", name);
-    return;
+    strcpy(last_ls_name, ls_to_draw);
+    p=parse_lsystem_path();
+    loaded_lsystems=ls_load_all(p);
+    if (!default_lsystem) {
+      status=STATUS_CALLING_ERROR;
+      g_print("No lsystem named '%s' found.\n", last_ls_name);
+    }
+    selected_lsystem=default_lsystem;
     break;
   case RUN_WITH_LAST_VALS:
-    g_print ("With last vals: %s\nNot supported yet!\n", name);
-    return;
+    gimp_get_data(PLUG_IN_NAME, &last_ls_name);
+    p=parse_lsystem_path();
+    loaded_lsystems=ls_load_all(p);
+    if (!default_lsystem) {
+      status=STATUS_CALLING_ERROR;
+      g_print("Strange, no lsystem named '%s' found.\n", last_ls_name);
+    }
+    selected_lsystem=default_lsystem;
     break;
   default:
     break;
@@ -163,24 +195,75 @@ void run (gchar * name,
   }
 
   if (status == STATUS_SUCCESS) {
-    gimp_progress_init ("Lsystem compilation...");
-
     rets=gimp_run_procedure("gimp_undo_push_group_start", &nrets,
 			    PARAM_IMAGE, image,
 			    PARAM_END);
     if (rets[0].data.d_status != STATUS_SUCCESS) 
       g_print("No UNDO will be available.\n");
     gimp_destroy_params(rets, nrets);
+    
+    /*
+    rets=gimp_run_procedure("gimp_image_disable_undo", &nrets,
+			    PARAM_IMAGE, image,
+			    PARAM_END);
+    if (rets[0].data.d_status != STATUS_SUCCESS) 
+      g_print("No UNDO will be available.\n");
+    gimp_destroy_params(rets, nrets);
+    */
 
     compiled=lsystem_create_string(&selected_lsystem->def);
-    gimp_progress_update(1.0);
-    gimp_progress_init("Lsystem rendering...");
+    
+    if (selected_lsystem->prm.autoscale) {
+      selected_lsystem->prm.steplen=1.0;
+      keep_x=selected_lsystem->prm.initpos.x;
+      keep_y=selected_lsystem->prm.initpos.y;
+      selected_lsystem->prm.initpos.x=0.0;
+      selected_lsystem->prm.initpos.y=0.0;
+      maxpos.x=-MAXFLOAT;
+      maxpos.y=-MAXFLOAT;
+      minpos.x=MAXFLOAT;
+      minpos.y=MAXFLOAT;
+
+      srand(selected_lsystem->prm.seed);
+      lsystem_preview(compiled, &selected_lsystem->prm);
+      free_slut(&selected_lsystem->prm);
+
+      kx=(gdouble)image_wid / (maxpos.x - minpos.x);
+      ky=(gdouble)image_hei / (maxpos.y - minpos.y);
+      k=kx < ky ? kx : ky;
+      k*=selected_lsystem->prm.as_correction; /* Adjusting scale according 
+						 to user-specified correction */
+      selected_lsystem->prm.steplen *= k;
+      if (gimp_debug()) 
+	g_print("Scaling factor=%f, steplen=%f\n", k, selected_lsystem->prm.steplen);
+
+      selected_lsystem->prm.initpos.x -= minpos.x;
+      selected_lsystem->prm.initpos.y -= minpos.y;
+      selected_lsystem->prm.initpos.x *= k;
+      selected_lsystem->prm.initpos.y *= k;
+      /* Centering the result on the selection/image */
+      selected_lsystem->prm.initpos.x+= (image_wid - (maxpos.x-minpos.x)*k)/2; 
+      selected_lsystem->prm.initpos.y+= (image_hei - (maxpos.y-minpos.y)*k)/2; 
+      /* Adjusting to the origin of selection */
+      selected_lsystem->prm.initpos.x += image_ox;    
+      selected_lsystem->prm.initpos.y += image_oy;
+    }
+
+    sprintf(last_ls_name, "Rendering %s", selected_lsystem->def.name);
+    gimp_progress_init(last_ls_name);
+    srand(selected_lsystem->prm.seed);
     lsystem_render(compiled, &selected_lsystem->prm);
     gimp_progress_update(1.0);
-    
+
+    if (selected_lsystem->prm.autoscale) {
+      selected_lsystem->prm.initpos.x=keep_x;
+      selected_lsystem->prm.initpos.y=keep_y; 
+    }   
+
     if (run_mode == RUN_INTERACTIVE) {
       strncpy(last_ls_name, selected_lsystem->def.name, 255);
       gimp_set_data(PLUG_IN_NAME, &last_ls_name, sizeof(last_ls_name));
+      ls_write_all();
     }
 
     rets=gimp_run_procedure("gimp_undo_push_group_end", &nrets,
@@ -188,6 +271,12 @@ void run (gchar * name,
 			    PARAM_END);
     gimp_destroy_params(rets, nrets);
 
+    /*
+    rets=gimp_run_procedure("gimp_image_enable_undo", &nrets,
+			    PARAM_IMAGE, image,
+			    PARAM_END);
+    gimp_destroy_params(rets, nrets);
+    */
 
     if (run_mode != RUN_NONINTERACTIVE)
       gimp_displays_flush ();
@@ -220,9 +309,10 @@ gint ls_display_dialog ()
   gchar **argv;
   guchar *color_cube;
 
-  argc = 1;
-  argv = g_new (gchar *, 1);
+  argc = 2;
+  argv = g_new (gchar *, 2);
   argv[0] = g_strdup ("lsystem");
+  argv[1] = g_strdup ("--show-events");
 
   gtk_init (&argc, &argv);
   gtk_rc_parse (gimp_gtkrc ());
@@ -240,7 +330,6 @@ gint ls_display_dialog ()
   p=parse_lsystem_path();
   loaded_lsystems = ls_load_all (p);
   window = ls_main_new ();
-  ls_create_lsystems_list ();
   gtk_main ();
   return do_draw;
 }
@@ -277,6 +366,8 @@ LSinfo *ls_lsystem_new()
   prm->srules[1]=g_strdup("-:-30:" CURRENT_BRUSH ":-1:-1:0:");
   prm->steplen=5;
   prm->stepfactor=50;
+  prm->autoscale=TRUE;
+  prm->as_correction=1.0;
   def->iters=4;
   return ret;
 }
@@ -317,6 +408,10 @@ gint lsystem_write_to_file (LSinfo * s)
   gint i;
   gchar *tmp, *ext;
 
+  if (!s->dirty) {
+    if (gimp_debug()) g_print("%s is clean, not writing.\n", s->def.name);
+    return 1;
+  }
   if (s->def.fname[0] != '/') { /* File name of new lsystem. 
 				   Path will be supplied */
     if (!lsystems_home_path) {
@@ -344,7 +439,7 @@ gint lsystem_write_to_file (LSinfo * s)
     g_print("Writing to file: '%s'\n", s->def.fname);
   f=fopen(s->def.fname, "w");
   if (!f) {
-    g_error ("Cannot open L-System file '%s' for writing!\n", s->def.fname);
+    g_print ("Cannot open L-System file '%s' for writing!\n", s->def.fname);
     return 0;
   }
   fprintf (f, "LSname:%s\n", s->def.name);
@@ -371,6 +466,8 @@ gint lsystem_write_to_file (LSinfo * s)
   fprintf (f, "step_random:%f\n", s->prm.step_random);
   fprintf (f, "acu_step_random:%d\n", s->prm.acu_step_random);
   fprintf (f, "random_seed:%d\n", s->prm.seed);
+  fprintf (f, "autoscale:%d\n", s->prm.autoscale ? 1 : 0 );
+  fprintf (f, "as_correction:%f\n", s->prm.as_correction);
   fprintf (f, "END.\n");
   fclose (f);
   return 1;
@@ -418,9 +515,12 @@ LSinfo *lsystem_read_from_file (gchar * name)
     return NULL;
   }
   ret->def.fname = g_strdup (name);
+  ret->prm.autoscale=TRUE;
+  ret->prm.as_correction=1.0;
   while (!feof (f)) {
     fgets (line, MAX_LINE_LENGTH, f);
     wrd = strtok (line, ":\n");
+    if (wrd==NULL) goto error;
     if (strcmp (wrd, "LSname") == 0) {
       ret->def.name = g_strdup (strtok (NULL, ":\n"));
       continue;
@@ -557,6 +657,20 @@ LSinfo *lsystem_read_from_file (gchar * name)
       ret->prm.seed = atol (wrd);
       continue;
     }
+    if (strcmp (wrd, "autoscale") == 0) {
+      wrd = strtok (NULL, "\n");
+      if (!wrd)
+	goto error;
+      ret->prm.autoscale = atoi (wrd);
+      continue;
+    }
+    if (strcmp (wrd, "as_correction") == 0) {
+      wrd = strtok (NULL, "\n");
+      if (!wrd)
+	goto error;
+      ret->prm.as_correction = atof (wrd);
+      continue;
+    }
     if (strcmp (wrd, "END.") == 0) {
       break;
     }
@@ -659,6 +773,14 @@ GList *parse_lsystem_path ()
   return path_list;
 }
 
+gint lscmp(LSinfo *a, LSinfo *b)
+{
+  if (!a) return -1;
+  if (!b) return 1;
+  return strcmp(a->def.name, b->def.name);
+}
+
+
 /*
    Load all lsystems, which are found in lsystems-path-list. 
  */
@@ -706,10 +828,11 @@ GList *ls_load_all (GList * list)
 
 	if (!err && S_ISREG (filestat.st_mode)) {
 	  lsystem = lsystem_read_from_file (filename);
-	  if (lsystem)
-	    g_list_append (lslist, lsystem);
-	  if (lsystem->def.name && (strcmp(lsystem->def.name, last_ls_name)==0))
-	    selected_lsystem=lsystem;
+	  if (lsystem) {
+	    lslist=g_list_insert_sorted (lslist, lsystem, (GCompareFunc) lscmp);
+	    if (lsystem->def.name && (strcmp(lsystem->def.name, last_ls_name)==0))
+	      default_lsystem=lsystem;
+	  }
 	}
 	g_free (filename);
       }				/* while */
@@ -821,7 +944,7 @@ static void change_color (guchar r[3])
   gimp_palette_set_foreground(r[0], r[1], r[2]);
 }
 
-static Path *new_path (Position p)
+Path *new_path (Position p)
 {
   Path *path = (Path *) g_malloc (sizeof (Path));
 
@@ -831,7 +954,7 @@ static Path *new_path (Position p)
   return add_point (path, p);
 }
 
-static Path *add_point (Path * path, Position p)
+Path *add_point (Path * path, Position p)
 {
   static gint delta = 50;
   gdouble *tail;
@@ -868,6 +991,30 @@ static void flush_path (Path * path)
   g_free (path);
 }
 
+
+/* L-system previewing (for BBox calculations */
+
+inline Path *prv_add_point (Path * path, Position p)
+{
+  if (p.x < minpos.x) minpos.x=p.x;
+  if (p.y < minpos.y) minpos.y=p.y;
+  if (p.x > maxpos.x) maxpos.x=p.x;
+  if (p.y > maxpos.y) maxpos.y=p.y;
+
+  return NULL;
+}
+
+inline Path *prv_new_path (Position p)
+{
+  return prv_add_point(NULL, p);
+}
+
+static void prv_flush_path (Path * path)
+{
+  g_assert(path==NULL);
+}
+
+
 gchar *
   lsystem_create_string (Lsystem * s)
 {
@@ -895,9 +1042,9 @@ gchar *
     pptr = p;
     for (j = 0; j < strlen (p); j++) {
       if (lut[(guint) *pptr] == NULL) {
-	if (ravail == 0) {
-	  r = (gchar *) g_realloc (r, rlen + delta);
-	  ravail = delta;
+	if (ravail < 2) {
+	  r = (gchar *) g_realloc (r, rlen + delta + 1);
+	  ravail = delta + 1;
 	  delta *= 1.5;
 	}			/* Endif */
 	r[rlen++] = *pptr;
@@ -906,7 +1053,7 @@ gchar *
       } else {
 	addstr = lut[(guint) *pptr];
 	addlen = strlen (addstr);
-	if (ravail < addlen) {
+	if (ravail < addlen+1) {
 	  r = (gchar *) g_realloc (r, rlen + delta + addlen);
 	  ravail = delta + addlen;
 	  delta *= 1.5;
@@ -921,8 +1068,8 @@ gchar *
     p = r;
     r = (gchar *) g_malloc (1);
     r[0] = 0;
-    gimp_progress_update((1.0/s->iters)*i);
-    /*    g_print("ITER %d, rlen=%d, ravail=%d, delta=%d\n", i, rlen, ravail, delta);
+    /*
+    g_print("ITER %d, rlen=%d, ravail=%d, delta=%d\n", i, rlen, ravail, delta);
     fflush (stdout);
     */
     rlen = 0;
@@ -940,9 +1087,9 @@ gchar *
     pptr = p;
     for (j = 0; j < strlen (p); j++) {
       if (lut[(guint) *pptr] == NULL) { /* Simple copying of character */
-	if (ravail == 0) {
-	  r = (gchar *) g_realloc (r, rlen + delta);
-	  ravail = delta;
+	if (ravail < 2) {
+	  r = (gchar *) g_realloc (r, rlen + delta + 1);
+	  ravail = delta + 1;
 	  delta *= 1.5;
 	}			/* Endif */
 	r[rlen] = *pptr;
@@ -951,7 +1098,7 @@ gchar *
       } else { /* Inserting text replacing the character */
 	addstr = lut[(guint) *pptr];
 	addlen = strlen (addstr);
-	if (ravail < addlen) {
+	if (ravail < addlen + 1) {
 	  r = (gchar *) g_realloc (r, rlen + delta + addlen);
 	  ravail = delta + addlen;
 	  delta *= 1.5;
@@ -1016,7 +1163,7 @@ Special *ls_parse_srule(gchar *srule)
 int lsystem_render (gchar * s, Rparams * p)
 {
   register gchar *sptr;
-  Position pos;
+  Position pos, keep_pos;
   Special **slut;
   Special *spec, **akt;
   Path *path;
@@ -1032,7 +1179,7 @@ int lsystem_render (gchar * s, Rparams * p)
   if (!real_begin) { 
     real_begin=s;
     real_length=strlen(real_begin);
-    every_count=real_length/50;
+    every_count=real_length/30;
     increment=1.0/real_length;
   }
   if (p->slut != NULL)
@@ -1081,15 +1228,17 @@ int lsystem_render (gchar * s, Rparams * p)
       break;
     case '(':			/* Begining of a "sub l-system" */
       p->steplen *= p->stepfactor;
+      keep_pos=p->initpos;
       p->initpos = pos;
       scanned = lsystem_render (sptr + 1, p);
+      p->initpos=keep_pos;
       p->steplen /= p->stepfactor;
       sptr += 1 + scanned;
       break;
     default:			/* Special cases or nothing. */
-      if (slut[(guint) *sptr] != NULL) {
+      if (slut[(guchar) *sptr] != NULL) {
 	flushed = 0;
-	spec = slut[(guint) *sptr];
+	spec = slut[(guchar) *sptr];
 	pos.angle += spec->turn_angle +
 	  rand_norm (p->turn_random) * spec->turn_angle;
 	if (spec->brush_name || spec->brush_spc >= 0 ||
@@ -1117,9 +1266,84 @@ int lsystem_render (gchar * s, Rparams * p)
   set_drawing_context (saved_context);
 
   return (sptr - s);
+
 }
 
-static void free_slut (Rparams * p)
+int lsystem_preview (gchar * s, Rparams * p)
+{
+  register gchar *sptr;
+  Position pos, keep_pos;
+  Special **slut;
+  Special *spec, **akt;
+  Path *path;
+  gint scanned, i, flushed;
+  gfloat slen;
+
+  if (p->slut != NULL)
+    slut = (p->slut);
+  else {
+    p->slut = (Special **) g_malloc0 (256 * sizeof (Special *));
+    slut = (p->slut);
+    for (i = 0; i < p->srulnum; i++) {
+      akt = &slut[(guint) p->srules[i][0]];
+      (*akt)=ls_parse_srule(p->srules[i]);
+    }
+  }
+
+  sptr = s;
+  pos=p->initpos;
+
+  path = prv_new_path (pos);
+  while ((*sptr != 0) && (*sptr != ')')) {
+    switch (*sptr) {
+    case 'F':			/* Forward move */
+      if (p->acu_step_random)
+	slen = p->steplen += rand_norm (p->step_random)
+	  * p->steplen;
+      else
+	slen = p->steplen + rand_norm (p->step_random)
+	  * p->steplen;
+      pos.x += (gfloat) slen *cos (PI * pos.angle / 180.0);
+      pos.y += (gfloat) slen *sin (PI * pos.angle / 180.0);
+      path = prv_add_point (path, pos);
+      break;
+    case 'f':			/* Forward jump */
+      if (p->acu_step_random)
+	slen = p->steplen += rand_norm (p->step_random)
+	  * p->steplen;
+      else
+	slen = p->steplen + rand_norm (p->step_random)
+	  * p->steplen;
+      prv_flush_path (path);
+      pos.x += (gfloat) slen *cos (PI * pos.angle / 180.0);
+      pos.y += (gfloat) slen *sin (PI * pos.angle / 180.0);
+      path = prv_new_path (pos);
+      break;
+    case '(':			/* Begining of a "sub l-system" */
+      p->steplen *= p->stepfactor;
+      keep_pos=p->initpos;
+      p->initpos = pos;
+      scanned = lsystem_preview (sptr + 1, p);
+      p->steplen /= p->stepfactor;
+      p->initpos=keep_pos;
+      sptr += 1 + scanned;
+      break;
+    default:			/* Special cases or nothing. */
+      if (slut[(guchar) *sptr] != NULL) {
+	flushed = 0;
+	spec = slut[(guchar) *sptr];
+	pos.angle += spec->turn_angle +
+	  rand_norm (p->turn_random) * spec->turn_angle;
+      }
+    }				/* End of switch(...) */
+    sptr++;
+  }				/* End of while(...) */
+  if (path) prv_flush_path(path);
+  return (sptr - s);
+}
+
+
+void free_slut (Rparams * p)
 {
   gint i;
   Special **slut;
@@ -1131,6 +1355,7 @@ static void free_slut (Rparams * p)
 	g_free (slut[i]);
     }				/* Endfor */
   }				/* Endif  */
+  p->slut=NULL;
 }
 
 
@@ -1143,9 +1368,9 @@ static gfloat roufast ()
   static gfloat okrdou = RAND_MAX;
 
   do {
-    u = random ();
+    u = rand();
     u = (u + 1.0) / okrdou;
-    x = random ();
+    x = rand();
     x = 2.0 * sqrt (2.0 / exp (1.0)) * (x + 1.0) / okrdou - sqrt (2.0 / exp (1.0));
     x = x / u;
     if (x * x <= 2.0 * (3.0 - u * (4.0 + sqrt (u))))
@@ -1158,7 +1383,7 @@ static gfloat roufast ()
   return (x);
 }
 
-static gfloat rand_norm (gfloat sigma)
+gdouble rand_norm (gdouble sigma)
 {
   return roufast () * sigma;
 }
